#!/bin/bash
#set -x
#Globals
MAPR_HOME="${MAPR_HOME:-__PREFIX__}"
drillBase="${MAPR_HOME}/drill"
drill_secure="false"
drill_qs_only=0
drill_only=0
drill_qs_and_drill=0
zkNodesList=""
drillQSEnabled=0
drillBitsNodeOn=0
drillQSNodeOn=0

if [ -e "${MAPR_HOME}/server/common-ecosystem.sh" ]; then
    . "${MAPR_HOME}/server/common-ecosystem.sh"
else
   echo "Failed to source common-ecosystem.sh"
   exit 0
fi



function ParseCommonOpts() {
    local i
    # for now we only care about the ZK stuff
    # which we really don't care about because we get it from 
    # getZKServers() function
    # this is here just for reference
    while [[ $# > 0 ]] ; do
        echo "\$1=$1"
        echo "\$#=$#"
        case "$1" in
        -zk) zkNodesList=$2;
              shift 2;;
        -OT)   
              OTNodesList="$2";
              echo "ParseCommonOpts: $*"
              shift 2;;
        *)
              echo "ParseCommonOpts-i: $i";
              echo "ParseCommonOpts1: $*";
              shift;;
        esac
    done
}

function GetDrillVersion() {
  local ver=""
  if [ -f "${drillBase}/drillversion" ]; then
    ver=$(cat "${drillBase}/drillversion")
  else
    ver=`ls -t ${drillBase} | sed 's/^.*drill-//' | head -1 | awk '{print $1}'`
  fi
  echo "$ver"
}

function ConfigureRunUserForDrill() {
  local DRILL_DIR CURR_USER CURR_GROUP
  DRILL_DIR=$1
  if [ ! -d $DRILL_DIR/conf ]; then
    return
  fi

  CURR_USER=$2
  CURR_GROUP=$3
  chown -R $CURR_USER:$CURR_GROUP "${DRILL_DIR}/conf" >> $logFile 2>&1
}

function getMemValueFromWardenFile() {
    local rc
    local wFile=$1
    local wIdent=$2
    local wVariable=$3
    local RES=$(cat $wFile | grep "^$wIdent" | sed -e 's/.*'"$wVariable="'\([0-9]*[GgMmKk]*\).*$/\1/')
    rc=$?
    echo "$RES"
    return $rc
}

function ConfigureDrillBitsRole() {
  local numThreads=4
  local heapMem=""
  local rpcStanza="\n"
  local oldClusterId="drillbits1"
  local oldZkConnect="localhost:2181"
  local newClusterId="${clusterName}-drillbits"


  if ! hasRole 'drill-bits' ; then
    logInfo "Skipping Drill Bits Role configuration... Not found"
    return
  else
    drillBitsNodeOn=1
  fi

  logInfo "Configuring Drill Bits Role"
  local DRILL_BASE_DIR=${drillBase}/drill-${DRILL_VER}

  ConfigureRunUserForDrill $DRILL_BASE_DIR $MAPR_USER $MAPR_GROUP

  local DRILL_CONF_DIR="${DRILL_BASE_DIR}/conf"
  local drillConf="${DRILL_CONF_DIR}/drill-override.conf"
  local drillDistribEnv="${DRILL_CONF_DIR}/distrib-env.sh"
  local drillDistribConf="${DRILL_CONF_DIR}/drill-distrib.conf"
  local drillDistribQSMemConf="${DRILL_CONF_DIR}/drill-distrib-mem-qs.conf"

  if [ -e "$drillDistribConf" -a "$drillQSEnabled" -eq 0 ]; then
    if hasRole 'drill-qs' ; then
       # -QS option was already used - keep using it
       drillQSEnabled=1
       drillQSNodeOn=1
    fi
  fi

  if grep -q "cluster-id: \"$oldClusterId\"" $drillConf; then
    # we have a new install with default information - remove it
    sed -i -e "/cluster-id: \"$oldClusterId\"/d" $drillConf
    sed -i -e "/zk.connect: \"$oldZkConnect\"/d" $drillConf
  fi

  # always configure the number of threads required for QS
  if [ -e "$drillDistribQSMemConf" ]; then
    numThreads=$(grep DrillRpcUserThreads "$drillDistribQSMemConf" | cut -d'=' -f2)
    if [ -z "$numThreads" ]; then
      numThreads="4"
    fi
  fi
  rpcStanza="\n\trpc {\n\t\tuser.client.threads: $numThreads\n\t}\n"

  if [ "$drillQSEnabled" -eq 1 ]; then
    touch "${MAPR_HOME}/roles/drill-qs"
    if [ -e "$drillDistribQSMemConf" ]; then
      heapMem=$(grep DrillQSWardenHeapTotal "$drillDistribQSMemConf" | cut -d'=' -f2)
    fi
  fi

  # TODO improve this to remove drill.exec stanza only instead of overwriting file
  echo -e "drill.exec {\n\tcluster-id: \"$newClusterId\",\n\tzk.connect: \"$zkNodesList\"${rpcStanza}}\n" > $drillDistribConf

  drillEnvExistingExport=`grep "export HADOOP_HOME" ${drillDistribEnv}`
  if [ -z "$drillEnvExistingExport" ]; then
    echo "" >> $drillDistribEnv
    echo "export HADOOP_HOME=/opt/mapr/hadoop/hadoop-${hadoopVersion}" >> $drillDistribEnv
  else
    sed -i -e "s/^export HADOOP_HOME=.*/export HADOOP_HOME=\/opt\/mapr\/hadoop\/hadoop-${hadoopVersion}/g" $drillDistribEnv
  fi

  local wardenPkgDrillConf="${DRILL_CONF_DIR}/warden.drill-bits.conf"
  local wardenInstalledDrillConf="${MAPR_CONF_DIR}/warden.drill-bits.conf"
  local wardenTmpDrillConf="/tmp/warden.drill-bits.conf"
  local wardenConfFileInstalled=0
  local conf_drill_heap=$(getMemValueFromWardenFile $wardenConfFile service.env DRILL_HEAP)
  local conf_drill_mdm=$(getMemValueFromWardenFile $wardenConfFile service.env DRILL_MAX_DIRECT_MEMORY)
  local conf_drill_dccs=$(getMemValueFromWardenFile $wardenConfFile service.env DRILLBIT_CODE_CACHE_SIZE)
  local conf_drill_heap_min=$(getMemValueFromWardenFile $wardenConfFile service.heapsize.min service.heapsize.min)
  local conf_drill_heap_max=$(getMemValueFromWardenFile $wardenConfFile service.heapsize.max service.heapsize.max)
  local def_drill_heap=$(getMemValueFromWardenFile $wardenPkgDrillConf service.env DRILL_HEAP)
  local def_drill_mdm=$(getMemValueFromWardenFile $wardenPkgDrillConf service.env DRILL_MAX_DIRECT_MEMORY)
  local def_drill_dccs=$(getMemValueFromWardenFile $wardenPkgDrillConf service.env DRILLBIT_CODE_CACHE_SIZE)
  local def_drill_heap_min=$(getMemValueFromWardenFile $wardenPkgDrillConf service.heapsize.min service.heapsize.min)
  local def_drill_heap_max=$(getMemValueFromWardenFile $wardenPkgDrillConf service.heapsize.max service.heapsize.max)
  local def_qs_heap=$(getMemValueFromWardenFile $drillDistribQSMemConf 'export DRILL_HEAP' DRILL_HEAP)
  local def_qs_mdm=$(getMemValueFromWardenFile $drillDistribQSMemConf 'export DRILL_MAX_DIRECT_MEMORY' DRILL_MAX_DIRECT_MEMORY)
  local def_qs_dccs=$(getMemValueFromWardenFile $drillDistribQSMemConf 'export DRILLBIT_CODE_CACHE_SIZE' DRILLBIT_CODE_CACHE_SIZE)
  local def_qs_heap_min=$(getMemValueFromWardenFile $drillDistribQSMemConf 'DrillQSWardenHeapTotal' DrillQSWardenHeapTotal)
  local def_qs_heap_max=$def_qs_heap_min
  local new_drill_heap=""
  local new_drill_mdm=""
  local new_drill_dccs=""
  local new_drill_heap_min=""
  local new_drill_heap_max=""
  local updateMemSettings=0

  if [ -e "$wardenInstalledDrillConf" ]; then
    cp "$wardenInstalledDrillConf" "$wardenTmpDrillConf"
    wardenConfFileInstalled=1
  else
    cp "$wardenPkgDrillConf" "$wardenTmpDrillConf"
  fi
  wardenConfFile="$wardenTmpDrillConf"
  # configure memory requirement
  if [ "$drillQSEnabled" -eq 0 ]; then
      new_drill_heap=$def_drill_heap
      new_drill_mdm=$def_drill_mdm
      new_drill_dccs=$def_drill_dccs
      new_drill_heap_min=$def_drill_heap_min
      new_drill_heap_max=$def_drill_heap_max
  else
      new_drill_heap="${def_qs_heap}m"
      new_drill_mdm="${def_qs_mdm}m"
      new_drill_dccs="${def_qs_dccs}m"
      new_drill_heap_min="$def_qs_heap_min"
      new_drill_heap_max="$def_qs_heap_max"
  fi
  # only change if memory settings are either the QS defaults or Drill defaults
  # if customer has changed them, leave as is
  if [ \( "$conf_drill_heap" = "${def_qs_heap}m" -o "$conf_drill_heap" = "$def_drill_heap" \) -a \
       \( "$conf_drill_mdm" = "${def_qs_mdm}m" -o "$conf_drill_mdm" = "$def_drill_mdm" \) -a \
       \( "$conf_drill_dccs" = "${def_qs_dccs}m" -o "$conf_drill_dccs" = "$def_drill_dccs" \) -a \
       \( "$conf_drill_heap_min" = "${def_qs_heap_min}" -o "$conf_drill_heap_min" = "$def_drill_heap_min" \) -a \
       \( "$conf_drill_heap_max" = "${def_qs_heap_max}" -o "$conf_drill_heap_max" = "$def_drill_heap_max" \) ]; then
      updateMemSettings=1
  fi
  if [ "$updateMemSettings" -eq 1 ]; then
      sed -i -e "s/DRILL_HEAP=[0-9]*[GgMmKkBb]/DRILL_HEAP=$new_drill_heap/" $wardenTmpDrillConf
      sed -i -e "s/DRILL_MAX_DIRECT_MEMORY=[0-9]*[GgMmKkBb]/DRILL_MAX_DIRECT_MEMORY=$new_drill_mdm/" $wardenTmpDrillConf
      sed -i -e "s/DRILLBIT_CODE_CACHE_SIZE=[0-9]*[GgMmKkBb]/DRILLBIT_CODE_CACHE_SIZE=$new_drill_dccs/" $wardenTmpDrillConf
      sed -i -e "s/service.heapsize.min=[0-9]*[GgMmKkBb]*/service.heapsize.min=$new_drill_heap_min/" $wardenTmpDrillConf
      sed -i -e "s/service.heapsize.max=[0-9]*[GgMmKkBb]*/service.heapsize.max=$new_drill_heap_max/" $wardenTmpDrillConf
  fi

  if [ "$drillBitsNodeOn" -eq 1 ]; then
    if [ "$wardenConfFileInstalled" -eq 0 ]; then
      if ! [ -d "${MAPR_CONF_DIR}" ]; then
        mkdir -p "${MAPR_CONF_DIR}"
      fi
    fi
    cp "$wardenTmpDrillConf" "$wardenInstalledDrillConf"
  fi
}

# Main

#sets MAPR_USER/MAPR_GROUP/logfile
initCfgEnv

#get opts
drill_usage="usage: $0 [-EC <common opts>] [--secure|--unsecure|--customSecure ] [-qsOnly] [-qsDrill] [-drillOnly]"
if [ ${#} -gt 1 ] ; then
    OPTS=`getopt -a -o h -l EC: -l secure: -l unsecure -l customSecure -l qsOnly -l qsDrill -l drillOnly -- "$@"`
    if [ $? != 0 ] ; then
        echo ${drill_usage}
        exit 2
    fi
    eval set -- "$OPTS"
    
    for i ; do
        case "$i" in
            --EC)
                eco_common_opts=$(echo $2 | sed "s/^' *//;s/ *'\$//");
                #eco_common_opts="$2";
                shift 2;;
            --customSecure)
                drill_secure="true";
                shift 1;;
            -h)
                echo ${drill_usage}
                exit 2
                ;;
            --drillOnly)
                drill_only=1;
                shift 1;;
            --qsDrill)
                drill_qs_and_drill=1;
                shift 1;;
            --qsOnly)
                drill_qs_only=1;
                shift 1;;
            --secure)
                drill_secure="custom";
                shift 1;;
            --unsecure)
                drill_secure="false";
                shift 1;;
            --)
                shift;;
        esac
    done
else
    echo "${drill_usage}"
    exit 2
fi

ParseCommonOpts $eco_common_opts
zkNodesList=$(getZKServers)
if [ $? -ne 0 -o -z "$zkNodesList" ]; then
    logInfo "WARNING: Skipping Drill Bits Role configuration... zkNodesList not set"
    exit
fi

echo "OTNodesList: $OTNodesList"
clusterName=$(getClusterName)
DRILL_VER=$(GetDrillVersion)
ConfigureDrillBitsRole
