#!/usr/bin/env bash

. /lib/lsb/init-functions

NAME="mesos-dns"
DESC="mesos-dns"

BASEMAPR=${MAPR_HOME:-/opt/mapr}
PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin
CONFIG=${config:-/opt/mapr/conf/config.json}
LOG_DIR=${log_dir:-/var/log/mesos}
DNS_ARGS="-config=$CONFIG -log_dir=$LOG_DIR"
DNSINITLOG=${BASEMAPR}/logs/mesos-dns.log

PID=${BASEMAPR}/pid/mesos-dns.pid

start() {
  start-stop-daemon --start --background --quiet \
                      --pidfile "$PID" --make-pidfile \
                      --exec $BASEMAPR/mesos-services/mesos-services-0.1/bin/mesos-dns -- $DNS_ARGS >> $DNSINITLOG 2>&1 || return 1
}

stop() {
  start-stop-daemon --stop --quiet --pidfile "$PID"
}

case "${1-}" in
  start)
     echo -n "Starting $DESC: "
     start
     echo "$NAME."
     ;;
  stop)
     echo -n "Stopping $DESC: "
     stop
     echo "$NAME."
     ;;
  restart)
     echo -n "Restarting $DESC: "
     stop
     sleep 1
     start
     echo "$NAME."
     ;;
  status)
     status_of_proc -p "$PID" "$NAME" "$NAME" && exit 0 || exit $?
     ;;
  *)
     echo "Usage: $0 {start|stop|restart|status}" >&2
     exit 1
     ;;
 esac
