#!/usr/bin/env bash

# Source function library.
. /etc/init.d/functions

. /lib/lsb/init-functions

NAME="mesos-dns"
DESC="mesos-dns"
command="mesos-dns"

BASEMAPR=${MAPR_HOME:-/opt/mapr}
PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin
CONFIG=${config:-/opt/mapr/mesos-services/mesos-services-0.1/conf/config-dns.json}
LOG_DIR=${log_dir:-/var/log/mesos}
DNS_ARGS="-config=$CONFIG -log_dir=$LOG_DIR"
SCRIPTNAME=`readlink -f $0`
if [ -z "$SCRIPTNAME" ]; then
 DIRNAME=`dirname $0`
else
 DIRNAME=`dirname $SCRIPTNAME`
fi

DNSINITLOG=${DIRNAME}/../logs/mesos-dns-init.log

PIDFILE=${BASEMAPR}/pid/mesos-dns.pid

start() {
  if [ -f $PIDFILE ]; then
    if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
      echo $command running as process `cat $PIDFILE`. Stop it
      exit 1
    fi
  fi

  $BASEMAPR/mesos-services/mesos-services-0.1/bin/mesos-dns $DNS_ARGS >> $DNSINITLOG 2>&1 &
  echo $! > $PIDFILE

  sleep 1
  if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
   echo "Starting $command, logging to $log"
  else
    echo "Error: $command can not be started."
    exit 1
  fi


}

stop() {
    #
    # REMINDER: If the parameters for killproc should
    # change in this function, then the parameters need
    # to be verified against SuSE in order to maintian
    # support.
    #
    local delay="-d 10"

    if [ -f "/etc/SuSE-release" ]; then
        delay="-t 10"
    fi

    #
    # killproc needs -p before -d !
    #
    if [ -f "${PIDFILE}" ]; then
        killproc -p ${PIDFILE} ${delay} ${NAME}
        killproc ${delay} ${NAME}
    else
        killproc ${delay} ${NAME}
    fi

    rm -f ${PIDFILE}

  return 0
}

case "${1-}" in
  start)
     echo -n "Starting $DESC: "
     start
     echo "$NAME."
     ;;
  stop)
     echo -n "Stopping $DESC: "
     stop
     echo "$NAME."
     ;;
  restart)
     echo -n "Restarting $DESC: "
     stop
     sleep 1
     start
     echo "$NAME."
     ;;
  status)
     status -p "$PIDFILE" "$NAME"
     ;;
  *)
     echo "Usage: $0 {start|stop|restart|status}" >&2
     exit 1
     ;;
 esac
