#!/bin/bash

# Source function library.
. /etc/init.d/functions

. /lib/lsb/init-functions

NAME="marathon"
DESC="marathon"
command="marathon"

BASEMAPR=${MAPR_HOME:-/opt/mapr}
PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin
PIDFILE=${BASEMAPR}/pid/marathon.pid
DIRNAME=`dirname $0`
MARINITLOG=${DIRNAME}/../logs/marathon-init.log

start() {
  # exit if it is already running
  if [ -f $PIDFILE ]; then
    if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
      echo $command running as process `cat $PIDFILE`. Stop it
      exit 1
    fi
  fi

  /usr/bin/marathon >> $MARINITLOG 2>&1 &
  echo $! > $PIDFILE

  sleep 1
  if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
   echo "Starting $command, logging to $log"
  else
    echo "Error: marathon can not be started."
    exit 1
  fi

}

stop() {
  if [ -f $PIDFILE ]; then
    if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
      echo stopping $command
      kill `cat $PIDFILE`
      while kill -0 `cat $PIDFILE` > /dev/null 2>&1; do
          sleep 1;
      done
      return 0
    else
      echo $command not running.
      return 1
    fi
  else
    echo $command not running.
    return 1
  fi
}

do_status() {
  if [ -f $PIDFILE ]; then
    if kill -0 `cat $PIDFILE` > /dev/null 2>&1; then
      echo $command running as process `cat $PIDFILE`.
      return 0
    fi
      echo $PIDFILE exists with pid `cat $PIDFILE` but no $command.
      return 1
  fi
  echo $command not running.
  return 1
}

case "${1-}" in
  start)
    echo -n "Starting $DESC: "
    start
    echo "$NAME."
    ;;
  stop)
    echo -n "Stopping $DESC: "
    stop
    echo "$NAME."
    ;;
  restart)
    echo -n "Restarting $DESC: "
    stop
    sleep 1
    start
    echo "$NAME."
    ;;
  status)
    do_status
    ;;
  *)
    echo "Usage: $0 {start|stop|restart|status}" >&2
    exit 1
    ;;
esac

